*  Directory for SAS code, data and output - remember trailing slash;

%let SAMPDIR=C:\HOW\DelGobbo\;

*  Library containing the sample data;

libname sample "&SAMPDIR" access=read;

*  Library to store styles and tagsets;

libname mylib "&SAMPDIR";

*  Set the ODS search path for styles and tagsets;

ods path mylib.tmplmst(update)
         sashelp.tmplmst(read);

*  Include recent version of the ExcelXP ODS tagset;

%include "&SAMPDIR.ExcelXP.sas";

*  Create a custom style based on the stock "sansPrinter" style;

proc template;
  define style styles.XLsansPrinter;
    parent=styles.sansprinter;
    
    /*  Change attributes of the column headings  */
    
    style header from header /
      font_size = 10pt
      just      = center
      vjust     = bottom;
      
    /*  Change attributes of the blank lines  */

    style notecontent from notecontent /
      bordertopwidth    = 1
      borderbottomwidth = 1
      borderleftwidth   = 1
      borderrightwidth  = 1;

    /*  Used to create bold text  */

    style data_bold from data /
      font_weight = bold;

    /*  Used to center text  */

    style data_center from data /
      just = center;

    /*  Used to convert SAS date values to Excel date values  */

    style data_date9 from data /
      tagattr = 'type:DateTime format:ddmmmyyyy';
  end;
run; quit;

*  Create a SAS table that is used for the range flags worksheet;

data Legend;
length LabFlag 8 Range $30;
LabFlag = 0; Range = 'Normal';                       output;
LabFlag = 1; Range = 'Low';                          output;
LabFlag = 3; Range = 'Clinically Significant Low';   output;
LabFlag = 2; Range = 'High';                         output;
LabFlag = 4; Range = 'Clinically Significant High';  output;
label Range   = 'Range Flags'
      LabFlag = 'Color';
run;

*;
*  Create a format for traffic lighting, based
*  on the lab flag values
*
*  0 : white    Normal
*  1 : #CCFFFF  Low
*  3 : #FFCC00  Clinically Significant Low
*  2 : #9999FF  High
*  4 : #FF6600  Clinically Significant High
*;

proc format;
  value FlagFmt  0 = 'white'
                 1 = '#CCFFFF'
                 3 = '#9999FF'
                 2 = '#FFCC00'
                 4 = '#FF6600';
run; quit;

ods listing close;
ods tagsets.ExcelXP path="&SAMPDIR" file='LabReport.xml' style=XLsansprinter;

title;
footnote;

*  Create the range flage worksheet;

ods tagsets.ExcelXP options(sheet_name='Range Flags');

proc print data=Legend noobs label;
  var Range;
  var LabFlag / style(Column)=[foreground=FlagFmt. background=FlagFmt.];
run; quit;

*  Need to reset the option value;

ods tagsets.ExcelXP options(sheet_name='none'); 

*  Create the lab results worksheets;

ods tagsets.ExcelXP options(sheet_interval='bygroup' sheet_label='16.2'
                            suppress_bylines='yes'
                            absolute_column_width='15,7,7,7,7,10,10,10,10,10,10,10,10,10,10'
                            autofit_height='yes'
                            frozen_headers='yes'
                            frozen_rowheaders='5');

proc report data=sample.LabResults split='*' nowindows;
by protocol;

*  'ID' columns;

column Treat Patient LabSite Visit VisitDate;

*  Data columns with spanned headers;

column ('Haematology' Haemoglobin_Result Haematocrit_Result RBC_Result WBC_Result
         ('Differential' Lymphocytes_Result Monocytes_Result Neutrophils_Result
                         Eosinophils_Result Basophils_Result Platelets_Result
         )
       );

*  Hidden columns containing the range flags;

column Haemoglobin_Flag Haematocrit_Flag RBC_Flag WBC_Flag
       Lymphocytes_Flag Monocytes_Flag Neutrophils_Flag Eosinophils_Flag
       Basophils_Flag Platelets_Flag;


*  Dummy column to perform traffic lighting;

column dummy;

*  'ID' columns;

define Treat     / display order style(Column)=data_bold;
define Patient   / display order style(Column)=data_center;
define LabSite   / display order style(Column)=data_center;
define Visit     / display order style(Column)=data_center;
define VisitDate / display       style(Column)=data_date9;

*  Data columns;

define Haemoglobin_Result / display;
define Haematocrit_Result / display;
define RBC_Result         / display;
define WBC_Result         / display;
define Lymphocytes_Result / display;
define Monocytes_Result   / display;
define Neutrophils_Result / display;
define Eosinophils_Result / display;
define Basophils_Result   / display;
define Platelets_Result   / display;

*  Hidden columns containing the range flags;

define Haemoglobin_Flag / display noprint;
define Haematocrit_Flag / display noprint;
define RBC_Flag         / display noprint;
define WBC_Flag         / display noprint;
define Lymphocytes_Flag / display noprint;
define Monocytes_Flag   / display noprint;
define Neutrophils_Flag / display noprint;
define Eosinophils_Flag / display noprint;
define Basophils_Flag   / display noprint;
define Platelets_Flag   / display noprint;

*  Dummy column to perform traffic lighting;

define dummy / computed noprint;

*  Traffic light the data columns based on the hidden columns;

compute dummy;

  array name(10) $31 ('Haemoglobin_Result' 'Haematocrit_Result' 'RBC_Result' 
                      'WBC_Result' 'Lymphocytes_Result' 'Monocytes_Result' 
                      'Neutrophils_Result' 'Eosinophils_Result' 'Basophils_Result'
                      'Platelets_Result');

  array flag(10) Haemoglobin_Flag Haematocrit_Flag RBC_Flag WBC_Flag 
                 Lymphocytes_Flag Monocytes_Flag Neutrophils_Flag Eosinophils_Flag
                 Basophils_Flag Platelets_Flag;

  *;
  *  Loop over all the _Result columns ('name' array), and set the 
  *  BACKGROUND style attribute based on the value of the corresponding 
  *  _Flag column ('flag' array).
  *;

  do i = 1 to dim(name);
    if (flag(i) gt 0)
      then call define(name(i), 
                       'style', 
                       'style=[background=' ||
                         put(flag(i), FlagFmt.) || 
                         ']');
  end;

endcomp;

*  Insert a blank line between treatments;

compute after Treat;
  line ' ';
endcomp;

run; quit;

ods tagsets.ExcelXP close;